//NOTE: Given weapons don't delete themselves on death or respawn.
// The following event need to be in your game for this to clean the weapons up.
// However, there can only be one event listener stored total, so in order to not conflict with another you need the cleanup function inside it.

//"post_inventory_application" event is sent when a player gets a whole new set of items, aka touches a resupply locker / respawn cabinet or spawns in.
// Add "hPlayer.GTFW_Cleanup" to your script to clean weapons up.
// -> Remember to change the handle.
function OnGameEvent_post_inventory_application(params)
{
	if ("userid" in params)
	{
		local hPlayer = GetPlayerFromUserID(params.userid)
		hPlayer.GTFW_Cleanup()	//<-Add this to your own post_inventory_application event. It must be at the beginning!
	}
}
	__CollectEventCallbacks(this, "OnGameEvent_", "GameEventCallbacks", RegisterScriptGameEventListener)
	
// The prior event should be deleted so it doesn't conflict with another `post_inventory_application` event. //
// Include the rest of this script! //

IncludeScript("give_tf_weapon_all.nut")	// tables for all TF2 weapons
//IncludeScript("give_tf_weapon_custom.nut") is at the very bottom of the script to fix parsing issues
Convars.SetValue("tf_dropped_weapon_lifetime", 0)	//disables dropped weapons because they're buggy with this script

//vscript cvars. Adjust how you like, to customize the script easily.
::CVAR_GTFW_GIVEWEAPON_REPLACE_WEAPONS <- true	//if true, overwrites current weapon in slot. True by default. NOTE: Cannot use more than two weapons in a slot, unless using "hud_fastswitch 0".
::CVAR_GTFW_USE_VIEWMODEL_FIX <- true	//Automatically fixes any and all viewmodel arms to match the class you're playing as. True by default.
::CVAR_GTFW_GIVEWEAPON_AUTO_SWITCH <- true // Automatically switches weapon to another if giving a weapon. True by default.
::CVAR_GTFW_DELETEWEAPON_AUTO_SWITCH <- true // Automatically switches weapon to another if deleting a weapon. True by default.
::CVAR_GTFW_DISABLEWEAPON_AUTO_SWITCH <- true // Automatically switches weapon to another if disabling a weapon. True by default.
::CVAR_GTFW_ENABLEWEAPON_AUTO_SWITCH <- false // Automatically switches weapon to another if re-enabling a weapon. False by default.
::CVAR_GTFW_DEBUG_MESSAGES <- false // Prints debug messages, like ERROR messages from this script if it fails. False by default.

/*vScript "give_tf_weapon" (originally giveweapon.nut) framework for Team Fortress 2, by Yaki
Special thanks ficool2 for finding the netprops to make the first weapon giving function, AddWeapon().
Special thanks Mr.Burguers (TF2Maps) for being a wealth of knowledge, teaching me how to use vscript
Special thanks devs of Super Zombie Fortress + SCP Secret Fortress for releasing their source code (taught me what netprops were needed for custom viewmodels)

Notes to Users
 - Set CVAR_GTFW_* for easy tweaking of script.
 - When using GiveWeapon, weapon strings for unlocks don't need "The " in them. (i.e. "The Sandvich" is invalid, but "Sandvich" is acceptable)
 --> Also accepts handles, as well as item index values (from items_game.txt)
 - GiveWeapon also returns the weapon as a handle. So you can add attributes to the weapon!
 - Given/Custom weapons don't delete themselves on respawn. 
 ->Add function `CTFPlayer.GTFW_Cleanup` to your `post_inventory_application` event to delete any unused weapons/viewmodels attached to the player.
 --> Please place at the beginning!
 --> Needs a player handle to clear weapons from.

PUBLIC OPERATIONS
 Made to make giving weapons, easy! Also supports custom weapons!
	CheckItems()
		Checks your current equipment, with item IDs. Spits out a list in chat with active weapon on top.
	CTFPlayer.GiveWeapon(weapon)
		Gives you a weapon. Any weapon, as long as the entity and ID match from items_game.txt.
		Returns given weapon as handle.
	CTFPlayer.GiveWeaponEx(searched_weapon, searched_itemID, new_weapon, new_itemID)
		Extended version of `CTFPlayer.GiveWeapon`.
		Searches the player's inventory for a weapon and replaces it.
		Can do exceptions for searched entities (i.e. Search every weapon classname EXCLUDING the one with this itemID)
	CTFPlayer.DeleteWeapon(weapon)
		Deletes a weapon from the player.
	CTFPlayer.DisableWeapon(weapon)
		Makes a weapon non-functional on player, meaning they can't switch to nor use it.
	CTFPlayer.EnableWeapon(weapon)
		Enables a weapon if it was disabled by the prior command
	CTFPlayer.ReturnWeapon(weapon)
		Searches weapon on player and returns as handle.
	CTFPlayer.ReturnWeaponTable(weapon)
		Find item on player by string, item ID, classname, slot, or handle. Else null.
	CTFPlayer.ReturnWeaponBySlot
		Finds an equip by slot. 0 = Primary, 1 = Secondary etc. Up to 6. Returns null if failed
	ReturnWeaponBySlotBool(weapon, slot)
		Finds equip if `weapon.GetSlot()` matches `slot` parameter. Returns true if found, false if else
	RegisterCustomWeapon(item_name, baseitem, stats_function, custom_weapon_model, custom_arms_model, custom_extra_wearable)
		Registers a custom weapon into the database.
		
		
PRIVATE OPERATIONS
 Functions used only in this file or debugging. Don't use unless you know what you're doing.
	CTFPlayer.AddWeapon(classname, itemindex, slot)
		Use if you want a specific weapon in a specific slot.
	CTFPlayer.UpdateArms(weapon, weapon_model, arms_model, arms_draw_seq)
		Updates the weapon's arm models
			Optional params: arms_model, arms_draw_seq
	CTFPlayer.CreateCustomWeaponModel(wearabletype, baseitem, weapon_model)
		Multiple functions. Creates tf_wearables and parents them to the player.
			wearabletype = bool, true for making tf_wearable, false for making tf_wearable_vm
			baseitem = your base item's handle. Accepts null or main_viewmodel handle for making class arms only.
			weapon_model = model.mdl
		Returns handle of the entity it creates
	CTFPlayer.CreateCustomWearable(weapon, wearable)
		Creates a `tf_wearable` and associates it with a weapon (if valid)
	CTFPlayer.HasGunslinger()
		Checks if wearer is using the Gunslinger.
	CTFPlayer.AddThinkToViewModel(classarms_model)
		Updates the think script on tf_viewmodel. Branches from `AddWeapon`.
	CTFPlayer.SwitchToActive(NewGun)
		Forcefully switches to given weapon. If null, switches to first weapon it finds.
		Essentially replaced by vscript built-in function Weapon_Switch
	GTFW_WepFix(exists)
		Updates classnames for shotguns, pistols, saxxy, etc, before creating the weapon.
		Returns param 'exists'
	GTFW_ReturnWeaponBySlotBool(weapon, slot)
		Same as `ReturnWeaponBySlotBool`, but param `slot` takes negatives instead.
	GTFW_SearchEquipReg(string_or_id)
		Find item by string or item ID. Used by `CTFPlayer.ReturnWeaponTable`
	CTFPlayer.SetCustomWeapon(item_name, baseitem, custom_weapon_model, custom_arms_model, stats_function)
		Sets a weapon after giving you a weapon as base.
		Was used for testing, but can be used. (Might add it back to public list)

Examples of Uses
	CheckItems()
		USE: In console, type "script CheckItems()" (without quotes)
			Shows slot, classname, and ID of all equipped weapons
	CTFPlayer.GiveWeapon(weapon_id_or_string)
		USE: hPlayer.GiveWeapon("Minigun")
			Gives player the stock minigun.
		USE: hPlayer.GiveWeapon(132)
			Gives player the Eyelander
	CTFPlayer.GiveWeaponEx(searched_weapon, searched_itemID, new_weapon, new_itemID)
		USE: hPlayer.GiveWeaponEx("tf_weapon_wrench", -7, "tf_weapon_knife", 4)
			Replaces all non-stock wrenches with stock knife
		USE: hPlayer.GiveWeaponEx("melee", -1, "tf_weapon_robot_arm", 142)
			Replaces all melees with Gunslinger
		USE: hPlayer.GiveWeaponEx("Brass Beast", -1, "Buff Banner", -1)
			Replaces just the Brass Beast with the Buff Banner.
	CTFPlayer.DeleteWeapon(weapon_classname_or_id_or_string)
		USE: hPlayer.DeleteWeapon("tf_weapon_minigun")
			Deletes only miniguns
		USE: hPlayer.DeleteWeapon("Minigun")
			Deletes only stock minigun
		USE: hPlayer.DeleteWeapon(15)
			Deletes only stock minigun with this ID
		USE: hPlayer.DeleteWeapon(TF_WEAPONSLOTS.PRIMARY)
			Deletes all primaries
		USE: hPlayer.DeleteWeapon(TF_WEAPONSLOTS.MELEE)	
			Deletes all melees
		USE: hPlayer.DeleteWeapon("misc") 
			Deletes all misc items (PDAs, watches, spellbooks, grappling hooks, etc)
	CTFPlayer.DisableWeapon(weapon_classname_or_id_or_string)
		USE: hPlayer.DisableWeapon("Vaccinator") 
			Disable only Vaccinator
		USE: hPlayer.DisableWeapon(TF_WEAPONSLOTS.SECONDARY) 
			Disable any secondary
		USE: hPlayer.DisableWeapon("tf_weapon_bottle") 
			Disables any tf_weapon_bottle entity
		USE: hPlayer.DisableWeapon(7)
			Disables only the stock Wrench (item ID#7)
	RegisterCustomWeapon(item_name, baseitem, stats_function, custom_weapon_model, custom_arms_model, custom_extra_wearable)
		USE: RegisterCustomWeapon("Hell Slayer", "Knife", CW_Stats_Example_Hell_Slayer_Knife, "models/weapons/c_models/c_bigaxe/c_bigaxe.mdl", "models/weapons/c_models/c_scout_arms.mdl", null)
			Makes "Hell Slayer", a stock knife using the Horsemann's REAL Headtaker, using Scout's bat animations. Stats non-functional as of v1
		USE: RegisterCustomWeapon("Big Bertha", "Grenade Launcher", CW_Stats_Example_Big_Bertha_GL, "models/workshop/weapons/c_models/c_demo_cannon/c_demo_cannon.mdl", "models/weapons/c_models/c_engineer_arms.mdl", null)
			Makes "Big Bertha", a grenade launcher using with "c_demo_cannon.mdl" (loose cannon model), using Engineer's animations. Stats non-functional as of v1
			
			
Notes/Clarification
		-> CTFPlayer = handle ID for your player's entity (aka player's serial number), called hPlayer all throughout examples.
	The following list are acceptable parameters for searching for weapons:
		-> Handles (aka the weapon ID's serial number)
		-> Classnames of weapons (i.e. "tf_weapon_wrench")
		-> items_game.txt weapon ID (i.e. 7) (7 is the stock wrench)
		-> Slots, using TF_WEAPONSLOTS with slot type. 
			-> TF_WEAPONSLOTS.PRIMARY, for example, gets all slots in the primary slot.
			Also acceptable:
			TF_WEAPONSLOTS.SECONDARY
			TF_WEAPONSLOTS.MELEE
			TF_WEAPONSLOTS.MISC = Anything that isn't melee, secondary, or primary (like spellbooks, grappling hooks, PDAs, watches, etc)
			TF_WEAPONSLOTS.SLOT3 = Engineer's "Build PDA", Spy's "Disguise Kit PDA"
			TF_WEAPONSLOTS.SLOT4 = Engineer's "Destroy PDA", Spy's watches
			TF_WEAPONSLOTS.SLOT5 = Engineer toolbox, sapper (stock are both "tf_weapon_builder", sapper unlocks are "tf_weapon_sapper")
			TF_WEAPONSLOTS.SLOT6 = spellbook, grappling hook. (PASSTIME gun?)
	itemID accepts:
		ANY: Valid items_game.txt IDs are 0 and above
		GiveWeaponEx: Can use exemption values by making the number negative (i.e. Wrench is itemID 7. Setting it to -7 would pick all wrenches except stock)
		GiveWeaponEx TIP: Use -0.0 to exempt all but all but Scout's stock bat (tf_weapon_bat)
		GiveWeaponEx TIP: Use any negative that isn't tied to that weapon to grab all weapons of that entity type. (i.e. searching "tf_weapon_wrench, -0.0," would search for any wrench)
	RegisterCustomWeapon Notes
		NOTE: Watch your map's ent count! It spawns 1 entity for the class_arms (tf_wearable_vm) and 1 for fixing metal on non-Engy classes. 
			It spawns 2 additional entities per weapon--one for the thirdperson model (tf_wearable), and another for firstperson (tf_wearable_vm)
		NOTE: the player's viewmodel has a think script to fix the weapon appearing when needed, and being invisible when not
			item_name = Custom item's name, to be called by GiveWeapon and other functions.
			baseitem = Accepts handles, weapon's entity classnames, weapons by slot (except "misc"), and weapon string name (i.e. "Crossbow")
			OPTIONAL: stats_function = custom stats to be called by a function. Set to 'null' if unused. (examples in give_tf_weapon_custom.nut)
			custom_weapon_model = your new model over the old one. Appears in thirdperson as well as firstperson.
			OPTIONAL: custom_arms_model = custom first person animations. Set to 'null' if unused.
			OPTIONAL: custom_extra_wearable = custom wearable cosmetic on wearer. Set to 'null' if unused.
	GiveWeaponEx
		searched_weapon = weapon to be replaced. See "searching for weapons" parameters for all acceptable args.
		searched_itemID = items_game.txt index. Negative numbers cause weapon to be excluded. If not a classname, ignores this value. Required when searching an entity classname (i.e. "tf_weapon_wrench").
		new_weapon = weapon to be given. Accepts: Weapon's string name (i.e. "Buff Banner"), or classname (i.e. "tf_weapon_knife")
		new_itemID = items_game.txt index. If new_weapon is a classname, must be a positive number. If not, it is ignored.


End instructions */

///-------------------------------------///
//function OnPostSpawn()
//{
	::PlayerLoadoutGlobal_DrawSeq <- {}
	::PlayerLoadoutGlobal_ClassArms <- {}
	::PlayerLoadoutGlobal_BonemergedArms <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_VM <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_TP <- {}
//}

PrecacheModel("models/weapons/c_models/c_engineer_gunslinger.mdl")

const GLOBAL_WEAPON_COUNT = 10

enum TF_AMMO
{
	NONE = 0
	PRIMARY = 1
	SECONDARY = 2
	METAL = 3
	GRENADES1 = 4 // e.g. Sandman, Jarate, Sandvich
	GRENADES2 = 5 // e.g. Mad Milk, Bonk,
	GRENADES3 = 6 // e.g. Spells
}

enum GTFW_ARMS
{
	SCOUT	=	"models/weapons/c_models/c_scout_arms.mdl",
	SNIPER	=	"models/weapons/c_models/c_sniper_arms.mdl",
	SOLDIER	=	"models/weapons/c_models/c_soldier_arms.mdl",
	DEMO	=	"models/weapons/c_models/c_demo_arms.mdl",
	MEDIC	=	"models/weapons/c_models/c_medic_arms.mdl",
	HEAVY	=	"models/weapons/c_models/c_heavy_arms.mdl",
	PYRO	=	"models/weapons/c_models/c_pyro_arms.mdl",
	SPY		=	"models/weapons/c_models/c_spy_arms.mdl",
	ENGINEER=	"models/weapons/c_models/c_engineer_arms.mdl",
	CIVILIAN=	"models/weapons/c_models/c_engineer_gunslinger.mdl",
}

::GTFW_MODEL_ARMS <-
[
	"models/weapons/c_models/c_medic_arms.mdl", //dummy
	"models/weapons/c_models/c_scout_arms.mdl",
	"models/weapons/c_models/c_sniper_arms.mdl",
	"models/weapons/c_models/c_soldier_arms.mdl",
	"models/weapons/c_models/c_demo_arms.mdl",
	"models/weapons/c_models/c_medic_arms.mdl",
	"models/weapons/c_models/c_heavy_arms.mdl",
	"models/weapons/c_models/c_pyro_arms.mdl",
	"models/weapons/c_models/c_spy_arms.mdl",
	"models/weapons/c_models/c_engineer_arms.mdl",
	"models/weapons/c_models/c_engineer_gunslinger.mdl",	//CIVILIAN/Gunslinger
]

::SearchBySlotsParameters <-
[
	-0.0,
	-1,
	-2,
	-3,
	-4,
	-5,
	-6,
]

::CTFPlayer.GTFW_Cleanup <- function()
{
	// Deletes any viewmodels that the script has created
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	AddThinkToEnt(main_viewmodel, null)	//clears script if it was being used
	if ( main_viewmodel != null && main_viewmodel.FirstMoveChild() != null )
	{
		main_viewmodel.FirstMoveChild().Kill()
	}
	main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[this.GetPlayerClass()] )
	for (local i = 0; i < 42; i++)
	{
		local wearable = Entities.FindByNameWithin(this, "tf_wearable_vscript", this.GetLocalOrigin(), 128)

		if ( wearable != null && wearable.GetOwner() == this )
		{
			if (CVAR_GTFW_DEBUG_MESSAGES) {
				printl("Deleting wearable... "+wearable+" Model# "+NetProps.GetPropInt(wearable, "m_nModelIndex") )
			}

			wearable.Kill()
		}
		else
		{
			break
		}
	}
//clears all custom weapons
	::PlayerLoadoutGlobal_CustomWeaponModels_VM[this] <- {}
	::PlayerLoadoutGlobal_CustomWeaponModels_TP[this] <- {}
	if ( this in PlayerLoadoutGlobal_BonemergedArms ) {
		delete PlayerLoadoutGlobal_BonemergedArms[this]
	}
	if ( this.GetPlayerClass() != Constants.ETFClass.TF_CLASS_ENGINEER ) {
		local metalammofix = this.CreateCustomWearable(null, "models/empty.mdl")	//Metal ammo fix. Creates invisible wearable w/ 2x metal attribute for non-Engineers
		metalammofix.AddAttribute("maxammo metal increased", 2, -1)
	}
	
}

::KillWepAll <- function(wep)
{
	local wearable = NetProps.GetPropEntity(wep, "m_hExtraWearable")
	if ( wearable != null ) {
		wearable.Kill()
	}
	wearable = NetProps.GetPropEntity(wep, "m_hExtraWearableViewModel")
	if ( wearable != null ) {
		wearable.Kill()
	}
	wep.Kill()
}

//FUNCTION: For testing if weapon is custom or is unintended
::IS_CUSTOM <- function(weapon)
{
	return NetProps.GetPropInt(weapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes")
}
::IS_UNINTENDED <- function(weapon)
{
	return NetProps.GetPropInt(weapon, "m_bValidatedAttachedEntity")
}

// FUNCTION: Finds a weapon based on handle, slot, ID, classnames, or string.
::CTFPlayer.ReturnWeaponTable <- function(weapon)
{
	local ItemID = null
	local baseitem = null
	
	if ( SearchAllWeapons.find(weapon) != null ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)

			if ( wep != null )
			{
				if ( wep.GetClassname() == weapon )
				{
					ItemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
					baseitem = GTFW_SearchEquipReg(ItemID)
					return baseitem
				}
			}
		}
	}
	else if ( SearchBySlotsParameters.find(weapon) != null ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep != null )
			{
				if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
				{
					ItemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
					baseitem = GTFW_SearchEquipReg(ItemID)
					return baseitem
				}
			}
		}
	}
	else {
		baseitem = GTFW_SearchEquipReg(weapon)
		ItemID = NetProps.GetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
		
		if ( baseitem != null ) {
			return baseitem
		} else if ( ItemID != null ) {
			baseitem = GTFW_SearchEquipReg(ItemID)
			if ( baseitem != null ) {
				return baseitem
			}
		}
	}
	if (CVAR_GTFW_DEBUG_MESSAGES) {
		printl("give_tf_weapon ERROR: ReturnWeaponTable failed. Returning null.")
	}
	return null
}

::GTFW_SearchEquipReg <- function(baseitem)
{
	local truefalse = true

	foreach (exists in TF_CUSTOM_WEAPONS_REGISTRY)
	{
		if ( exists.itemString == baseitem )
		{
			exists = GTFW_WepFix(exists)
			
			truefalse = false
			return exists
		}
	}
	if ( truefalse )
	{
		foreach (exists in TF_WEAPONS_ALL)
		{
			if ( exists.itemString == baseitem || exists.itemString2 == baseitem )
			{
				exists = GTFW_WepFix(exists)
				
				truefalse = false
				return exists
			}
			else if ( exists.itemID == baseitem || exists.itemID2 == baseitem )
			{
				if ( exists.itemID == baseitem ) {
					exists.itemID = exists.itemID
				}
				else if ( exists.itemID2 == baseitem ) {
					exists.itemID = exists.itemID2
				}
				
				exists = GTFW_WepFix(exists)
				
				truefalse = false
				return exists
			}
		}
		if ( truefalse )
		{
			foreach (exists in TF_WEAPONS_ALL_FESTIVE)
			{
				if ( exists.itemString == baseitem || exists.itemString2 == baseitem )
				{
					exists = GTFW_WepFix(exists)
					
					truefalse = false
					return exists
				}
				else if ( exists.itemID == baseitem )
				{
					exists = GTFW_WepFix(exists)

					truefalse = false
					return exists
				}
			}
			
			if ( truefalse )
			{
				foreach (exists in TF_WEAPONS_ALL_WARPAINTSnBOTKILLERS)
				{
					if ( exists.itemID == baseitem || exists.itemID2 == baseitem || exists.itemID3 == baseitem
					|| exists.itemID4 == baseitem || exists.itemID5 == baseitem || exists.itemID6 == baseitem
					|| exists.itemID7 == baseitem || exists.itemID8 == baseitem || exists.itemID9 == baseitem
					|| exists.itemID10 == baseitem || exists.itemID11 == baseitem || exists.itemID12 == baseitem
					|| exists.itemID13 == baseitem)
					{
						if ( exists.itemID == baseitem ) {
							exists.itemID = exists.itemID
						}
						else if ( exists.itemID2 == baseitem ) {
							exists.itemID = exists.itemID2
						}
						else if ( exists.itemID3 == baseitem ) {
							exists.itemID = exists.itemID3
						}
						else if ( exists.itemID4 == baseitem ) {
							exists.itemID = exists.itemID4
						}
						else if ( exists.itemID5 == baseitem ) {
							exists.itemID = exists.itemID5
						}
						else if ( exists.itemID6 == baseitem ) {
							exists.itemID = exists.itemID6
						}
						else if ( exists.itemID7 == baseitem ) {
							exists.itemID = exists.itemID7
						}
						else if ( exists.itemID8 == baseitem ) {
							exists.itemID = exists.itemID8
						}
						else if ( exists.itemID9 == baseitem ) {
							exists.itemID = exists.itemID9
						}
						else if ( exists.itemID10 == baseitem ) {
							exists.itemID = exists.itemID10
						}
						else if ( exists.itemID11 == baseitem ) {
							exists.itemID = exists.itemID11
						}
						else if ( exists.itemID12 == baseitem ) {
							exists.itemID = exists.itemID12
						}
						else if ( exists.itemID13 == baseitem ) {
							exists.itemID = exists.itemID13
						}
						
						exists = GTFW_WepFix(exists)

						return exists
					}
				}
			}
		}
	}
}
::GTFW_WepFix <- function(exists)
{
	if ( exists.className == "tf_weapon_saxxy" ) {
		exists.className = GTFW_Saxxy[this.GetPlayerClass()]
	}
	else if ( exists.className == "tf_weapon_pistol" ) {
		if ( this.GetPlayerClass() == 1 ) {
			exists.className = "tf_weapon_pistol_scout"
			exists.draw_seq = "p_draw"
			exists.reserve		= 36
		}
		else {
			exists.draw_seq = "pstl_draw"
			exists.reserve		= 200
		}
	}
	else if ( exists.className == "tf_weapon_shotgun" ) {
		if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_ENGINEER ) {
			exists.className = "tf_weapon_shotgun_primary"
			exists.draw_seq = "fj_draw"
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_HEAVYWEAPONS ) {
			exists.className = "tf_weapon_shotgun_hwg"
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_SOLDIER ) {
			exists.className = "tf_weapon_shotgun_soldier"
		}
		else if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_PYRO ) {
			exists.className = "tf_weapon_shotgun_pyro"
		}
	}
	if ( exists.className == "tf_weapon_katana" ) {
		if ( this.GetPlayerClass() == Constants.ETFClass.TF_CLASS_DEMOMAN ) {
			exists.draw_seq = "cm_draw"
		}
	}
	return exists
}

::CTFPlayer.HasWeaponClassname <- function(item_classname)
{
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep != null && wep.GetClassname() == item_classname ) {
			return true
		}
	}
	return false
}

//FUNCTION: Used to find switch to a gun forcefully. Don't use by itself.
// -> See top of script for info
::CTFPlayer.SwitchToActive <- function(NewGun)
{
	if ( NewGun == null ) {
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
		// pass if our active weapon was not the deleted one, nor any misc weapon like a PDA or spellbook
			if ( wep != null && wep != this.GetActiveWeapon() )
			{
				this.Weapon_Switch(wep)
				return wep
			}
		}
		return
	}
	else {
		this.Weapon_Switch(NewGun)
		local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
		main_viewmodel.EnableDraw()	//Fixes VM weapon not showing up when giving unintended weapon -> intended weapon to tfclass
		return NewGun
	}
}

// FUNCTION: Finds Gunslinger on player. If found, returns true.
::CTFPlayer.HasGunslinger <- function()
{
	if ( this.GetPlayerClass() == 9 )
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			local wepID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
			
			if ( wep != null && wep.GetClassname() == "tf_weapon_robot_arm" || wepID == 142 )
			{
				return true
			}
		}
	}
	return false
}

// FUNCTION: Used to update viewmodel arms. Don't use on its own
::CTFPlayer.UpdateArms <- function(weapon, weapon_model, arms_model, draw_anim)
{
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
// USE: Updates arms if the class holding it is the owner
// Checks if not a custom weapon, then does *not* use special VM fix for it.

	if ( IS_CUSTOM(weapon) == 0 && IS_UNINTENDED(weapon) == 0 )
	{
		if ( this.HasGunslinger() ) {
			PrecacheModel(GTFW_MODEL_ARMS[10])
			weapon.SetCustomViewModel( GTFW_MODEL_ARMS[10] )	//Gunslinger arms
		}
		else { 
			PrecacheModel(GTFW_MODEL_ARMS[this.GetPlayerClass()])
			weapon.SetCustomViewModel( GTFW_MODEL_ARMS[this.GetPlayerClass()] )
		}
		return arms_model
	}
	
// USE: This is the special VM fix.
// it creates class arms that bonemerge with VM, and adds the weapon in the VM as a tf_wearable_vm.
// It creates the world model, as well.
	local class_arms = null
	
	local weapon_id = NetProps.GetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
	local baseitem = this.GTFW_SearchEquipReg(weapon_id)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: UpdateArms failed. Returning null.")
		}
		return null
	}

	local baseitemClassname = baseitem.className
	local is_tfclass = baseitem.tf_class
	local draw_seq = baseitem.draw_seq
	
	if ( weapon.GetClassname() != "tf_wearable" && weapon.GetClassname() != "tf_wearable_demoshield"  && weapon.GetClassname() != "tf_wearable_razorback" ) {
		if ( CVAR_GTFW_USE_VIEWMODEL_FIX )
		{
		// adds a class arms model if nothing has parented to the main_viewmodel
		// this is used as a flag for initializing tables for the weapons
			if ( !( this in PlayerLoadoutGlobal_BonemergedArms ) ) {
				local wearable_handle = this.CreateCustomWeaponModel(false, main_viewmodel, GTFW_MODEL_ARMS[this.GetPlayerClass()])	// creates player class' arms, parents and does all that stuff
			// PURPOSE: So we don't create more arms than necessary
			// Puts into a table "player = arms model" for easy tracking
				::PlayerLoadoutGlobal_BonemergedArms[this] <- wearable_handle
			}
			class_arms = GTFW_MODEL_ARMS[is_tfclass]
			
			if ( is_tfclass == 0 )	//Is non-class
			{
				class_arms = GTFW_MODEL_ARMS[this.GetPlayerClass()]
			}
			else if ( is_tfclass == 9 && this.HasGunslinger() )	//Is Engineer with Gunslinger
			{
				class_arms = GTFW_MODEL_ARMS[10]
			}
			
		//updates class arms if not using new weapon's class arms
			if ( arms_model != null && draw_anim != null)
			{
				class_arms = arms_model
				main_viewmodel.SetModelSimple( class_arms )
				draw_seq = main_viewmodel.LookupSequence( draw_anim )
			}
			else
			{
				main_viewmodel.SetModelSimple( class_arms )
				draw_seq = main_viewmodel.LookupSequence( draw_seq )
			}
			
			if ( weapon_model != null )
			{
				local wearable_handle = this.CreateCustomWeaponModel(false, weapon, weapon_model)	// creates weapon in viewmodel, parents and does all that stuff
				
			//Mr. Burguers helped me with this part. Thanks Mr B!
				local wep = weapon.GetClassname()
				if ( !( this in PlayerLoadoutGlobal_CustomWeaponModels_VM ) ) {
					PlayerLoadoutGlobal_CustomWeaponModels_VM[this] <- {}
				}
				if ( wep in PlayerLoadoutGlobal_CustomWeaponModels_VM[this] )
				{
					local wepModel = PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep]
					if ( wepModel.IsValid() ) {
						KillWepAll(wepModel)
					}
					delete PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep]
				}
				PlayerLoadoutGlobal_CustomWeaponModels_VM[this][wep] <- wearable_handle


				
				if ( IS_CUSTOM(weapon) )	//only make third person model if weapon is custom
				{
					wearable_handle = this.CreateCustomWeaponModel(true, weapon, weapon_model)	// creates thirdperson/world model, parents and does all that stuff
					
					if ( !( this in PlayerLoadoutGlobal_CustomWeaponModels_TP ) ) {
						PlayerLoadoutGlobal_CustomWeaponModels_TP[this] <- {}
					}
					if ( wep in PlayerLoadoutGlobal_CustomWeaponModels_TP[this] )
					{
						local wepModel = PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep]
						if ( wepModel.IsValid() ) {
							KillWepAll(wepModel)
						}
						delete PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep]
					}
					PlayerLoadoutGlobal_CustomWeaponModels_TP[this][wep] <- wearable_handle
				}
			}
		}
	}

	return class_arms
}

// FUNCTION: Clears and updates the viewmodel think script for weapon switching, enabling/disabling visibility of custom weapons.

::CTFPlayer.AddThinkToViewModel <- function(armz)
{
	local class_arms = armz
	if ( CVAR_GTFW_USE_VIEWMODEL_FIX == false) {
		return
	}
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	local wearable_handle = null
		
		
	local Global_ClassArms = null
	local Global_DrawSeq = null
	if ( !( this in PlayerLoadoutGlobal_ClassArms ) ) {
		::PlayerLoadoutGlobal_ClassArms[this] <- {}
	}
	if ( !( this in PlayerLoadoutGlobal_DrawSeq ) ) {
		::PlayerLoadoutGlobal_DrawSeq[this] <- {}
	}
	
// FUNCTION: writes to the tables, which keep the class arms the weapon needs to update to when switched to,
// and the draw sequence. All draw sequences were found in the qc arms files, and are documented in give_tf_weapon_all.nut.
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		local wepID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
		
		if ( wep != null )
		{
			local baseitem = this.GTFW_SearchEquipReg(wepID)
			if ( baseitem != null) {
				PlayerLoadoutGlobal_DrawSeq[this][wep.GetClassname()] <- baseitem.draw_seq
				if ( baseitem.tf_class == this.GetPlayerClass() ||  baseitem.tf_class == 0 )
				{
					PlayerLoadoutGlobal_ClassArms[this][wep.GetClassname()] <- GTFW_MODEL_ARMS[this.GetPlayerClass()]
					wep.SetModelSimple( GTFW_MODEL_ARMS[this.GetPlayerClass()] )
				}
				else if ( this.HasGunslinger() )
				{
					PlayerLoadoutGlobal_ClassArms[this][wep.GetClassname()] <- GTFW_MODEL_ARMS[10]
					wep.SetModelSimple( GTFW_MODEL_ARMS[10] )
				}
				else if ( class_arms != null ) {
					PlayerLoadoutGlobal_ClassArms[this][wep.GetClassname()] <- class_arms
					wep.SetModelSimple( class_arms )
				}
				else {
					PlayerLoadoutGlobal_ClassArms[this][wep.GetClassname()] <- GTFW_MODEL_ARMS[baseitem.tf_class]
					wep.SetModelSimple( GTFW_MODEL_ARMS[baseitem.tf_class] )
				}
			}
		}
	}

// Think script itself.
// Reads from several tables to find weapon's draw sequences and class arms.
	if( main_viewmodel.ValidateScriptScope() )
	{
		local wep = null

		local player = this
		
		local entscriptname = "THINK_VM_FIX_"+this.tostring()
		
		local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
		
		local DisableDrawQueue = null
		local asdf = null
		
		const THINK_VMFIX_DELAY = 0
		
		local entityscript = main_viewmodel.GetScriptScope()
		entityscript[entscriptname] <- function()
		{
		// updates weapons' visibility based on which one is being used
			if ( player.GetActiveWeapon() != wep && !player.InCond(7) )
			{
				wep = player.GetActiveWeapon()
				local wepC = wep.GetClassname()
				
		// disables custom weapons, if the queue is full
				if ( DisableDrawQueue != null )
				{
					NetProps.SetPropInt(wep, "m_iState", 0)
					DoEntFire("!self", "RunScriptCode", "self.DisableDraw()", 0, null, DisableDrawQueue)	//using delay here purposely. Won't update b/c thinks too fast!!
					DisableDrawQueue = null
				}

			// This updates the viewmodel.
			// Remember, the weapon's sequences are decided by tf_viewmodel (or this handle, main_viewmodel)
				main_viewmodel.SetModelSimple( PlayerLoadoutGlobal_ClassArms[player][wep.GetClassname()] )
			//	main_viewmodel.SetSequence( main_viewmodel.LookupSequence( PlayerLoadoutGlobal_DrawSeq[player][wep.GetClassname()] ) )
				
			// These are vars are for the next part
				asdf = null
				DisableDrawQueue = null
				
			//Checks for custom weapon and/or weapon unintended for the class.
			//Passing as a custom weapon means it updates the weapon visibility, disables the base weapon,
			// then adds it to the queue to be disabled when switched away
				if ( IS_CUSTOM(wep) )
				{
					main_viewmodel.DisableDraw()		//makes firstperson weapon invisible (as well as other's class arms from the other class)
					if ( wepC in PlayerLoadoutGlobal_CustomWeaponModels_TP[player] ) {
						NetProps.SetPropInt(wep, "m_iState", 0)
						wep.EnableDraw()	//Have to send EnableDraw() first before DisableDraw(). Thanks Sorse!
						DoEntFire("!self", "RunScriptCode", "self.DisableDraw()", 0.01, null, wep)	//using delay here purposely. Won't update b/c thinks too fast!!
						
						asdf = PlayerLoadoutGlobal_CustomWeaponModels_TP[player][wepC]	//reads from table
						asdf.EnableDraw()
						DisableDrawQueue = asdf	//custom weapon disables visibility at beginning of think script.
					}
				}
				else if ( IS_UNINTENDED(wep) ) {
					main_viewmodel.DisableDraw()		//makes firstperson weapon invisible (as well as other's class arms from the other class)
				}
			// this part reads from the bonemerged arms table
			// who ever the class is, and which ever arms they are using, this will put it as a value...
				if ( this in PlayerLoadoutGlobal_BonemergedArms ) {
					local bonemerged = PlayerLoadoutGlobal_BonemergedArms[player]
					
					local wepID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
				// if ShortCircuit, disable the class arms' visibility
					if ( wepID == 528 ) { // Short Circuit
						bonemerged.DisableDraw()
					}
					else
					{
						bonemerged.EnableDraw()	//enable otherwise
					}
				}
				
				
				if (CVAR_GTFW_DEBUG_MESSAGES) {
					printl("WEAPON ACTIVE "+wep)
					printl( PlayerLoadoutGlobal_ClassArms[player][wep.GetClassname()] )
					printl( PlayerLoadoutGlobal_DrawSeq[player][wep.GetClassname()] )
				}
			}
			return THINK_VMFIX_DELAY
		}
		AddThinkToEnt(main_viewmodel, entscriptname)	//adds think script
	}
}

//Base for making weapons. Netprops compiled by ficool2.
::CTFPlayer.AddWeapon <- function(classname, itemindex, slot)
{
	local weapon = Entities.CreateByClassname(classname)

	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex", itemindex)
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iEntityQuality", 0)
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iEntityLevel", 1)
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_bInitialized", 1)
	//NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_iAccountID", -1)
	
	NetProps.SetPropInt(weapon, "m_bClientSideAnimation", 1)
	NetProps.SetPropInt(weapon, "m_fEffects", 129)
	//NetProps.SetPropInt(weapon, "m_iState", 2)
	NetProps.SetPropInt(weapon, "m_CollisionGroup", 11)
	
	local curTime = Time()
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flNextPrimaryAttack", curTime)
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flNextSecondaryAttack", curTime)
	NetProps.SetPropFloat(weapon, "LocalActiveWeaponData.m_flTimeWeaponIdle", curTime)
	
//	NetProps.SetPropInt(weapon, "m_bValidatedAttachedEntity", 1) //using as flag for if weapon is being used by unintended class
//	NetProps.SetPropInt(weapon, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 0) //setting to 1 means its custom
	NetProps.SetPropInt(weapon, "m_AttributeManager.m_iReapplyProvisionParity", 3)
	
	weapon.SetAbsOrigin(Vector(0,0,0))
	weapon.SetAbsAngles(QAngle(0,0,0))
	
	NetProps.SetPropInt(weapon, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropEntity(weapon, "m_hOwnerEntity", this)
	NetProps.SetPropEntity(weapon, "m_hOwner", this)
	weapon.SetOwner(null);
	weapon.SetOwner(this);
	
	Entities.DispatchSpawn(weapon)
	weapon.ReapplyProvision()
	
	/*NetProps.SetPropEntity(weapon, "m_AttributeManager.m_Item.m_iAccountID", this)
	printl ( NetProps.GetPropEntity(weapon, "m_AttributeManager.m_Item.m_iAccountID") )
	printl ( NetProps.GetPropEntity(weapon, "m_iAccountID") )
	printl ( NetProps.GetPropEntity(weapon, "m_Item.m_iAccountID") )*/
	
	local solidFlags = NetProps.GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	NetProps.SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags | Constants.FSolid.FSOLID_NOT_SOLID);
	
	solidFlags = NetProps.GetPropInt(weapon, "m_Collision.m_usSolidFlags");
	NetProps.SetPropInt(weapon, "m_Collision.m_usSolidFlags", solidFlags & ~(Constants.FSolid.FSOLID_TRIGGER));
	
	if ( classname != "tf_wearable_demoshield" || classname != "tf_wearable" || classname != "tf_wearable_razorback" )
	{
		NetProps.SetPropEntityArray(this, "m_hMyWeapons", weapon, slot)
	}
	weapon.SetLocalOrigin(this.GetLocalOrigin())
	weapon.SetLocalAngles(this.GetAbsAngles())
	DoEntFire("!self", "SetParent", "!activator", 0, this, weapon)
	NetProps.SetPropInt(weapon, "m_MoveType", 0)
	

//If added weapon is not intended to be used by player's tfclass, mark it as unintended for the class, to fix VM arms
	local addedWep = this.GTFW_SearchEquipReg(itemindex)
	if ( this.GetPlayerClass() != addedWep.tf_class )
	{
		NetProps.SetPropInt(weapon, "m_bValidatedAttachedEntity", 1)
	}

	if ( classname == "tf_weapon_builder")	//Engineer toolbox
	{
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_iObjectType", 0)
		NetProps.SetPropInt(weapon, "m_iSubType", 0)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.000", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.001", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.002", 1)
		NetProps.SetPropInt(weapon, "BuilderLocalData.m_aBuildableObjectTypes.003", 0)
	}
	
	local weapon_model = NetProps.GetPropInt(weapon, "m_iWorldModelIndex")
	
	local armz = this.UpdateArms(weapon, weapon_model, null, null)
	this.AddThinkToViewModel(armz)
	return weapon
}


/*FUNCTION: Finds an empty weapon equipment slot and gives weapon to the player
	-> See top of script for more info

 USE: hPlayer.GiveWeapon(15) // Gives the stock minigun
 USE: hPlayer.GiveWeapon("SMG")
*/

::CTFPlayer.GiveWeapon <- function(weapon)
{
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")
	AddThinkToEnt(main_viewmodel, null)	//clears script if it was being used
	main_viewmodel.SetModelSimple( GTFW_MODEL_ARMS[this.GetPlayerClass()] )
	
	
	local YourNewGunSaxtonApproved = null
	
//searches for the correct item based on parameter 'baseitem'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: GiveWeapon failed. Returning null.")
		}
		return null
	}
	local NewWeapon = baseitem.className
	local ItemID = baseitem.itemID
	local Slot = baseitem.slot
	local AmmoReserve = baseitem.reserve
	local Extra_Wearable = baseitem.wearable
	local draw_seq = baseitem.draw_seq
	
	if ( Extra_Wearable != null ) {
		this.CreateCustomWearable(weapon, Extra_Wearable)	// creates a tf_wearable and associates it with a weapon
	}

//searches inventory
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)

		if ( CVAR_GTFW_GIVEWEAPON_REPLACE_WEAPONS && wep != null ) {
			
			if ( wep.GetSlot() == Slot ) {
				KillWepAll(wep)
			}
		}
		if ( wep == null ) {
			YourNewGunSaxtonApproved = this.AddWeapon(NewWeapon, ItemID, i)
			break
		}
	}
	

// switches to another weapon if active one was deleted
// if given weapon matches the slot of the old weapon, stay in slot
	if (CVAR_GTFW_GIVEWEAPON_AUTO_SWITCH)
	{
		if ( Slot == YourNewGunSaxtonApproved.GetSlot() ) {
			this.SwitchToActive(YourNewGunSaxtonApproved)
		}
		else {
			this.SwitchToActive(null)
		}
	}
	
//Forcefully changes ammo type to primary if not Spy
	if ( NewWeapon == "tf_weapon_revolver" && this.GetPlayerClass != 8 )
	{
		NetProps.SetPropInt(YourNewGunSaxtonApproved, "LocalWeaponData.m_iPrimaryAmmoType", TF_AMMO.PRIMARY)
	}
	
// Fixes ammo.
	if ( YourNewGunSaxtonApproved.GetPrimaryAmmoType() != TF_AMMO.NONE ) {
		local aThreshold = 0
		if ( YourNewGunSaxtonApproved.GetPrimaryAmmoType() == TF_AMMO.PRIMARY ) {
			aThreshold = TF_AMMO_PER_CLASS_PRIMARY[this.GetPlayerClass()]
			YourNewGunSaxtonApproved.AddAttribute("hidden primary max ammo bonus", (AmmoReserve.tofloat() / aThreshold.tofloat()), -1)
		}
		else if ( YourNewGunSaxtonApproved.GetPrimaryAmmoType() == TF_AMMO.SECONDARY ) {
			aThreshold = TF_AMMO_PER_CLASS_SECONDARY[this.GetPlayerClass()]
			YourNewGunSaxtonApproved.AddAttribute("hidden secondary max ammo penalty", (AmmoReserve.tofloat() / aThreshold.tofloat()), -1)
		}
	//Sets ammo
		NetProps.SetPropIntArray(this, "m_iAmmo", AmmoReserve, YourNewGunSaxtonApproved.GetPrimaryAmmoType())
	}
	
// Purpose: Updates weapon to become custom if baseitem was found in custom weapon registry.
	if ( "class_arms" in baseitem )
	{
		local weapon_model = baseitem.model
		
	//set our weapon to custom only if our custom weapon has a non-stock model
		if ( weapon_model != null ) {
			NetProps.SetPropInt(YourNewGunSaxtonApproved, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1)	//marks our weapon as custom
		}
		local arms_model = baseitem.class_arms
		
		local armz = this.UpdateArms(YourNewGunSaxtonApproved, weapon_model, arms_model, draw_seq)
		this.AddThinkToViewModel(armz)
		
		local stats_function = baseitem.func
		if ( stats_function != null ) {
			stats_function(YourNewGunSaxtonApproved, this)
		}
	}
	
	return YourNewGunSaxtonApproved
}

/*FUNCTION: Deletes a weapon equipped on player
	-> See top of script for more info
 Accepts handles, entity classnames, strings, slots
*/

::CTFPlayer.DeleteWeapon <- function(weapon)
{
	local DeleteThis = weapon
	local Slot = null
//converts our weapon from string/ID to a classname...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: DeleteWeapon failed. Returning false.")
		}
		return false
	}
	DeleteThis = baseitem.className
	Slot = baseitem.slot
	
	local LOOPCOUNT_MAX = 1
	
	if ( weapon == "misc" || weapon == "MISC" || weapon == "Misc" )
	{
		LOOPCOUNT_MAX = 4	//setting to 3 for multiple PDAs, InvisWatch, Spellbook etc
	}
	for (local LOOPCOUNT_CURRENT = 0; LOOPCOUNT_CURRENT < LOOPCOUNT_MAX; LOOPCOUNT_CURRENT++)
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			local wep_itemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
			
			if ( wep != null )
			{
				if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
				{
					DeleteThis = wep.GetClassname()
				}
				if ( wep.GetClassname() == DeleteThis )
				{
					KillWepAll(wep)
					break
				}
			}
		}
	}
// switches to another weapon if active one was deleted
	if (CVAR_GTFW_DELETEWEAPON_AUTO_SWITCH) {
		if ( Slot == this.GetActiveWeapon().GetSlot() ) {
			this.SwitchToActive(null)
		}
	}
	
	return true
}


/*FUNCTION: Overwrites a weapon with another weapon
	-> See top of script for more info
 CTFPlayer = handle for your player's entity
 searched_weapon = weapon to be replaced. Accepts: "Primary", "Secondary", "Melee", or "Misc", weapon's string name (i.e. "Brass Beast"), or classname (i.e. "tf_weapon_wrench")
 searched_itemID = items_game.txt index. Negative numbers cause weapon to be excluded. If not a classname, ignores this value. Required for entity classnames (i.e. "tf_weapon_wrench").
 new_weapon = weapon to be given. Accepts: Weapon's string name (i.e. "Buff Banner"), or classname (i.e. "tf_weapon_knife")
 new_itemID = items_game.txt index. If new_weapon is a classname, must be positive. If not, it is ignored.

 USE: CTFPlayer.GiveWeaponEx("tf_weapon_wrench", -7, "tf_weapon_knife", 4)	//replaces all non-stock wrenches with stock knife
 USE: CTFPlayer.GiveWeaponEx("melee", -1, "tf_weapon_robot_arm", 142)	//replaces all melees with gunslinger
 USE: CTFPlayer.GiveWeaponEx("Brass Beast", -1, "Buff Banner", -1)		//replaces just the Brass Beast with the Buff Banner.
*/

::CTFPlayer.GiveWeaponEx <- function(searched_weapon, searched_itemID, new_weapon, new_itemID)
{
	local ReplaceThis = searched_weapon
//converts our weapon from string/ID to a classname...
	local baseitem = this.ReturnWeaponTable(searched_weapon)
	if ( baseitem != null ) {
		ReplaceThis = baseitem.className
	}
	
//putting "a" here so it doesn't error with nothing in array[0]
	local ReplaceThisID = searched_itemID
	local CheckIfAllButWeapon = split(ReplaceThisID.tostring()+"a",abs(ReplaceThisID).tostring())[0]
//a negative itemID is excluded in the search
	local ExcludeWeapon = false	//off by default
	if ( CheckIfAllButWeapon == "-" )	//if itemID is negative...
	{
		ReplaceThisID = abs(searched_itemID).tointeger()
		ExcludeWeapon = true
	}
//searches for the correct item based on parameter 'new_weapon'...
//returns data table
	baseitem = this.GTFW_SearchEquipReg(new_weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: GiveWeaponEx failed. Returning null.")
		}
		return null
	}
	local GiveThisID = baseitem.itemID
	
	local YourNewGunSaxtonApproved = null
	local LOOPCOUNT_MAX = 1
	
	if ( searched_weapon == "misc" || searched_weapon == "MISC" || searched_weapon == "Misc" )
	{
		LOOPCOUNT_MAX = 4	//setting to 3 for multiple PDAs, InvisWatch, Spellbook etc
	}
	for (local LOOPCOUNT_CURRENT = 0; LOOPCOUNT_CURRENT < LOOPCOUNT_MAX; LOOPCOUNT_CURRENT++)
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			local wep_itemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")
			
			if ( wep != null )
			{
				if ( GTFW_ReturnWeaponBySlotBool(searched_weapon, wep) )
				{
					ReplaceThis = wep.GetClassname()
				}
				if ( ( wep.GetClassname() == ReplaceThis && ExcludeWeapon == false && wep_itemID == ReplaceThisID ) || ( wep.GetClassname() == ReplaceThis && ExcludeWeapon == true && wep_itemID != searched_itemID ) )
				{
					KillWepAll(wep)
					if ( LOOPCOUNT_CURRENT == LOOPCOUNT_MAX - 1 )
					{
						YourNewGunSaxtonApproved = this.GiveWeapon(GiveThisID)
						break
					}
				}
			}
		}
	}
	
	return YourNewGunSaxtonApproved
}


/*	FUNCTION: Shows in console all equips on host of listen server.
		-> See top of script for more info

	USE: CheckItems()
*/

::CheckItems <- function()
{
	local ActiveWeapon = GetListenServerHost().GetActiveWeapon()
	local ActiveWeaponID = NetProps.GetPropInt(ActiveWeapon, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")

	Say(GetListenServerHost(), " ", false)
	Say(GetListenServerHost(), format("Active Slot%i [%s] (ItemID = %i)", ActiveWeapon.GetSlot(), ActiveWeapon.GetClassname(), ActiveWeaponID), false)
	
	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(GetListenServerHost(), "m_hMyWeapons", i)
		local wep_itemID = NetProps.GetPropInt(wep, "m_AttributeManager.m_Item.m_iItemDefinitionIndex")

		printl(i+" "+wep+" (ItemID = "+wep_itemID+")" )
			
		if ( wep != null && wep != ActiveWeapon)
		{
			Say(GetListenServerHost(), format("Slot%i [%s] (ItemID = %i)", wep.GetSlot(), wep.GetClassname(), wep_itemID), false)
		}
	}
}
/* FUNCTION: Returns weapon, searched by slot. 
Requested by Lizard of Oz
*/
::CTFPlayer.ReturnWeaponBySlot <- function(slot)
{
	if ( type( slot ) != "integer" ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: ReturnWeaponBySlot not an integer. Returning null.")
		}
		return null
	}
	local GetThis = null
	local YourGunFoundBySaxtonHale = null

	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep != null )
		{
			if ( ReturnWeaponBySlotBool(wep, slot) )
			{
				GetThis = wep.GetClassname()
			}
			if ( wep.GetClassname() == GetThis )
			{
				YourGunFoundBySaxtonHale = wep
				break
			}
		}
	}
	return YourGunFoundBySaxtonHale
}

::ReturnWeaponBySlotBool <- function(wep, slot)
{
	if ( type ( slot ) == "integer" ) {
		if ( wep.GetSlot() == slot ) {
			return true
		}
	}
	return false
}
::GTFW_ReturnWeaponBySlotBool <- function(wep, slot)
{
	if ( type ( slot ) == "integer" || slot == -0.0 ) {
		local asdf = split(slot.tostring()+"a",abs(slot).tostring())[0]
		if ( asdf == "-" )	//if slot checks as negative... test!
		{
			slot = abs(slot)
			if ( wep.GetSlot() == slot )
			{
				return true
			}
		}
	}
	return false
}


/* FUNCTION: disables being able to switch to a weapon.
	-> See top of script for more info
*/

::CTFPlayer.DisableWeapon <- function(weapon)
{
//searches for the correct item based on parameter 'weapon'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: DisableWeapon failed. Returning null.")
		}
		return null
	}
	
	local DisableThis = baseitem.className
	local Slot = baseitem.slot
	
	local BrokenGun = null
	local wep = null
	local LOOPCOUNT_MAX = 1
	
	if ( weapon == "misc" || weapon == "MISC" || weapon == "Misc" )
	{
		LOOPCOUNT_MAX = 4	//setting to 4 for multiple PDAs, InvisWatch, Spellbook etc
	}
	for (local LOOPCOUNT_CURRENT = 0; LOOPCOUNT_CURRENT < LOOPCOUNT_MAX; LOOPCOUNT_CURRENT++)
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep != null )
			{
				if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
				{
					DisableThis = wep.GetClassname()
				}
				if ( wep.GetClassname() == DisableThis )
				{
					if ( LOOPCOUNT_CURRENT == LOOPCOUNT_MAX - 1 )
					{
						BrokenGun = wep.__KeyValueFromString("classname", wep.GetClassname().tostring()+"_disabled") //works but visually buggy
						
						//wep.AddAttribute("disable weapon switch", 1, -1) //doesn't work??
						break
					}
				}
			}
		}
	}
	
// switches to another weapon if active one was disabled
	if (CVAR_GTFW_DISABLEWEAPON_AUTO_SWITCH) {
		if ( Slot == this.GetActiveWeapon().GetSlot() ) {
			this.SwitchToActive(null)
		}
	}
	
	return BrokenGun
}

/*//FUNCTION: Enables weapon if it was disabled.

	USE: hPlayer.EnableWeapon("primary")
*/
::CTFPlayer.EnableWeapon <- function(weapon)
{
	local FixThis = null
//searches for the correct item based on parameter 'weapon'...
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem != null ) {
		FixThis = baseitem.className
	}
	
	local FixedGun = null
	local LOOPCOUNT_MAX = 1
	
	if ( weapon == "misc" || weapon == "MISC" || weapon == "Misc" )
	{
		LOOPCOUNT_MAX = 4	//setting to 4 for multiple PDAs, InvisWatch, Spellbook etc
	}
	for (local LOOPCOUNT_CURRENT = 0; LOOPCOUNT_CURRENT < LOOPCOUNT_MAX; LOOPCOUNT_CURRENT++)
	{
		for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
		{
			local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
			
			if ( wep != null )
			{
				if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
				{
					FixThis = wep.GetClassname()
				}
				if ( wep.GetClassname() == FixThis )
				{
					if ( LOOPCOUNT_CURRENT == LOOPCOUNT_MAX - 1 )
					{
						for ( local CheckIfAllButWeapon = split(wep.GetClassname().tostring(),"_").len(); CheckIfAllButWeapon = CheckIfAllButWeapon[CheckIfAllButWeapon.len()]; ) {
							if ( CheckIfAllButWeapon == "disabled"  )
							{
								FixedGun = wep.__KeyValueFromString("classname", FixThis.tostring())
							}
						}
						break
					}
				}
			}
		}
	}
	
// switches to another weapon if active one was disabled

	if (CVAR_GTFW_ENABLEWEAPON_AUTO_SWITCH) {
		this.SwitchToActive(FixedGun)
	}
	
	return FixedGun
}

/* FUNCTION: Returns weapon from player as a hPlayer.
	-> See top of script for more info
*/

::CTFPlayer.ReturnWeapon <- function(searched_weapon)
{
	local GetThis = null
//searches for the correct item based on parameter 'searched_weapon'...
	local baseitem = this.ReturnWeaponTable(searched_weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: ReturnWeapon failed. Returning null.")
		}
		return null
	}
	GetThis = baseitem.className

	local YourGunFoundBySaxtonHale = null

	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep != null )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, searched_weapon) )
			{
				GetThis = wep.GetClassname()
			}
			if ( wep.GetClassname() == GetThis )
			{
				YourGunFoundBySaxtonHale = wep
				break
			}
		}
	}
	return YourGunFoundBySaxtonHale
}


/*Notes for viewmodels (UNFINISHED)
main_viewmodel (tf_viewmodel) needs to be the class arms model for bonemerging with weapon_arms. Cannot be unparented without complication. Cannot arbitrarily set "m_hWeapon" (netprop for current weapon entity seen by player).
weapon_arms (tf_wearable_vm) are the new class' hands that bonemerge with main_viewmodel.
baseitem (the weapon entity) displays the thirdperson model but the model it uses is the class arms.
When baseitem is unparented, it unparents the third person model from the player. However, doing this makes the weapon stay visible with taunts. (A think script fixes this)
baseitem's viewmodel animation sequences are taken from main_viewmodel.
baseitem's modelindex are the /class_arms/. Changing this changes what sequences are read by main_viewmodel.
baseitem netprop "m_iViewModelIndex" is viewmodel index. Cannot change arbitrarily.
baseitem netprop "m_iWorldModelIndex" is thirdperson index. Cannot change arbitrarily.

//How the weapon's VIEWMODEL changes visibility depending on when player is holding the weapon:
new_weapon_viewmodel (tf_wearable_vm) is another entity model parented to the existing viewmodel.
new_weapon_viewmodel links itself to the baseitem (w/ netprop "m_hWeaponAssociatedWith")
baseitem links itself to the new_weapon_viewmodel (w/ netprop "m_hExtraWearableViewModel")

//How the weapon's THIRDPERSON/WORLD MODEL changes visibility depending on when player is holding the weapon:
new_weapon_thirdperson (tf_wearable) is another entity model parented on the player.
new_weapon_thirdperson links itself to the baseitem (w/ netprop "m_hWeaponAssociatedWith")
baseitem links itself to the new_weapon_thirdperson (w/ netprop "m_hExtraWearable")

//viewmodel hierarchy:
main_viewmodel sequences [become]-> baseitem's sequences
main_viewmodel (tf_viewmodel) sequences are used by weapon_arms (tf_wearable_vm) for bonemerging
weapon_arms bonemerge with baseitem to complete the effect that class is holding the baseitem
baseitem's sequences MUST MATCH main_viewmodel's

//If you intend to update the weapon's animation sequences:
Can set Sequence of weapon via main_viewmodel. Example for Scout's Bat:
main_viewmodel.SetSequence(main_viewmodel.LookupSequence("b_draw") )

You can change which sequence list by changing the model itself to a different class arms model.
change the main_viewmodel using main_viewmodel.SetModelSimple() to change to new set of class arms
change the baseitem's modelindex (which are class_arms) to the animations you want to use for the new weapon
*/


::CTFPlayer.CreateCustomWeaponModel <- function(wearabletype, baseitem, weapon_model)
{
	local main_viewmodel = NetProps.GetPropEntity(this, "m_hViewModel")

	if ( type( weapon_model ) == "string" )
	{
		PrecacheModel(weapon_model)
		weapon_model = GetModelIndex(weapon_model)
	}
	if ( this.HasGunslinger() && baseitem == main_viewmodel ) {
		PrecacheModel(GTFW_MODEL_ARMS[10])
		weapon_model = GetModelIndex(GTFW_MODEL_ARMS[10])
	}
	
	local wearable_handle = null
	
// bool. If true, make a tf_wearable. If false, tf_wearable_vm.
	if (wearabletype)
	{
		wearable_handle = Entities.CreateByClassname("tf_wearable")
	}
	else
	{
		wearable_handle = Entities.CreateByClassname("tf_wearable_vm")
	}
	
	
// our properties. Taken from source code for Super Zombie Fortress + SCP Secret Fortress
	wearable_handle.SetAbsOrigin(this.GetLocalOrigin())
	wearable_handle.SetAbsAngles(this.GetLocalAngles())
	NetProps.SetPropInt(wearable_handle, "m_bValidatedAttachedEntity", 1)
	NetProps.SetPropInt(wearable_handle, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropInt(wearable_handle, "m_Collision.m_usSolidFlags", Constants.FSolid.FSOLID_NOT_SOLID)
	NetProps.SetPropInt(wearable_handle, "m_CollisionGroup", 11)
	NetProps.SetPropInt(wearable_handle, "m_fEffects", 129)
	
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_iEntityQuality", 0)
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_iEntityLevel", 1)
	NetProps.SetPropInt(wearable_handle, "m_AttributeManager.m_Item.m_bInitialized", 1)
	
	NetProps.SetPropEntity(wearable_handle, "m_hOwnerEntity", this)
	wearable_handle.SetOwner(this)
	
	NetProps.SetPropInt(wearable_handle, "m_nModelIndex", weapon_model)

// for tf_wearable (non-vm)
	if ( baseitem == null)
	{
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, this, wearable_handle)
	}
//for class arms
	else if ( baseitem == main_viewmodel)
	{
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, main_viewmodel, wearable_handle)
		this.EquipWearableViewModel(wearable_handle)
	}
// for viewmodel
	else if ( baseitem != null && wearable_handle.GetClassname() == "tf_wearable_vm" )
	{
		NetProps.SetPropEntity(wearable_handle, "m_hWeaponAssociatedWith", baseitem)
		NetProps.SetPropEntity(baseitem, "m_hExtraWearableViewModel", wearable_handle)
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, main_viewmodel, wearable_handle)
		this.EquipWearableViewModel(wearable_handle)
	}
//for world model (weapons only)
	else if ( baseitem != null && wearable_handle.GetClassname() == "tf_wearable" )
	{
	//	NetProps.SetPropEntity(wearable_handle, "m_hWeaponAssociatedWith", baseitem)
		NetProps.SetPropEntity(baseitem, "m_hExtraWearable", wearable_handle)
		Entities.DispatchSpawn(wearable_handle)
		DoEntFire("!self", "SetParent", "!activator", 0, this, wearable_handle)
		
	//	DoEntFire("!self", "AddOutput", "rendermode 1", 0, this, wearable_handle)
	//	DoEntFire("!self", "Color", "0 0 255", 0, this, wearable_handle)//colored weapons!
	//	DoEntFire("!self", "Clearparent", "", 0, null, baseitem)	//Disables the baseitem from appearing in thirdperson. However, various bugs like weapon appearing while taunting, or firing effecting origin messing up...
	}
// we name this for easy finding+cleanup
	wearable_handle.KeyValueFromString("targetname","tf_wearable_vscript")

	return wearable_handle
}

// FUNCTION: Creates a tf_wearable and associates it with a weapon

::CTFPlayer.CreateCustomWearable <- function(weapon, wearable_model)
{
	if ( type( wearable_model ) == "string" )
	{
		PrecacheModel(wearable_model)
		wearable_model = GetModelIndex(wearable_model)
	}
	
	local wearable = Entities.CreateByClassname("tf_wearable")
	
// our properties. Taken from source code for Super Zombie Fortress + SCP Secret Fortress
	wearable.SetAbsOrigin(this.GetLocalOrigin())
	wearable.SetAbsAngles(this.GetLocalAngles())
	NetProps.SetPropInt(wearable, "m_bValidatedAttachedEntity", 1)
	NetProps.SetPropEntity(wearable, "m_hOwnerEntity", this)
	NetProps.SetPropInt(wearable, "m_iTeamNum", this.GetTeam())
	NetProps.SetPropInt(wearable, "m_Collision.m_usSolidFlags", Constants.FSolid.FSOLID_NOT_SOLID)
	NetProps.SetPropInt(wearable, "m_CollisionGroup", 11)
	NetProps.SetPropInt(wearable, "m_fEffects", 129)
	
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iEntityQuality", 0)
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_iEntityLevel", 1)
	NetProps.SetPropInt(wearable, "m_AttributeManager.m_Item.m_bInitialized", 1)
	
	NetProps.SetPropInt(wearable, "m_nModelIndex", wearable_model)

	if ( weapon != null )
	{
		NetProps.SetPropEntity(wearable, "m_hWeaponAssociatedWith", weapon)
		NetProps.SetPropEntity(weapon, "m_hExtraWearable", wearable)
	}
	
	Entities.DispatchSpawn(wearable)
	DoEntFire("!self", "SetParent", "!activator", 0, this, wearable)
	wearable.KeyValueFromString("targetname", "tf_wearable_vscript")

	return wearable
}


/* FUNCTION: Sets a custom weapon model (!!!) on a player's weapon
 NOTE: Watch your map's ent count! It spawns 1 entity for the class_arms (tf_wearable_vm). It spawns 2 additional entities--one for the thirdperson model (tf_wearable), and another for firstperson (tf_wearable_vm)
 NOTE: Each weapon uses a think script to fix the weapon appearing when needed, and being invisible when not
*/

::CTFPlayer.SetCustomWeapon <- function(item_name, weapon, stats_function, custom_weapon_model, custom_arms_model, custom_extra_wearable)
{
//searches for the correct item based on parameter 'weapon'...
//'weapon' can be a handle, string name (ex "Brass Beast"), weapon ID (from items_game.txt), or slot (negative value)
	local baseitem = this.ReturnWeaponTable(weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: SetCustomWeapon failed. Returning null.")
		}
		return null
	}
	local CUSTOM_WEAPON = baseitem.className
	local ItemID = baseitem.itemID
	local draw_seq = baseitem.draw_seq
	

	for (local i = 0; i < GLOBAL_WEAPON_COUNT; i++)
	{
		local wep = NetProps.GetPropEntityArray(this, "m_hMyWeapons", i)
		
		if ( wep != null )
		{
			if ( GTFW_ReturnWeaponBySlotBool(wep, weapon) )
			{
				CUSTOM_WEAPON = wep.GetClassname()
			}
			if ( wep.GetClassname() == CUSTOM_WEAPON )
			{
				CUSTOM_WEAPON = wep
				break
			}
		}
	}
	
	local wearable = custom_extra_wearable
	if ( wearable != null ) {
		this.CreateCustomWearable(weapon, custom_extra_wearable)	// creates thirdperson/world model for wearables, parents and does all that stuff
	}
	
	if ( !(item_name in TF_CUSTOM_WEAPONS_REGISTRY) ) {
		this.RegisterCustomWeapon(item_name, CUSTOM_WEAPON, stats_function, custom_weapon_model, custom_arms_model, wearable)
	}
	if ( stats_function != null && type(stats_function) != "integer" && type(stats_function) != "string" && type(stats_function) != "bool" ) {
		stats_function(CUSTOM_WEAPON, this)
	}
	
	NetProps.SetPropInt(CUSTOM_WEAPON, "m_AttributeManager.m_Item.m_bOnlyIterateItemViewAttributes", 1)	//marks our weapon as custom
	local armz = this.UpdateArms(CUSTOM_WEAPON, custom_weapon_model, custom_arms_model, draw_seq)
	this.AddThinkToViewModel(armz)
	
//returns handle back to script for adding new attributes
	return CUSTOM_WEAPON
}

/* FUNCTION: Handles registering custom weapons. */

::RegisterCustomWeapon <- function(item_name, weapon, stats_function, custom_weapon_model, custom_arms_model, custom_extra_wearable)
{
//searches for the correct item based on parameter 'weapon'...
//'weapon' can be a handle, string name (ex "Brass Beast"), weapon ID (from items_game.txt), or slot (negative value)
	local baseitem = GTFW_SearchEquipReg(weapon)
	if ( baseitem == null ) {
		if (CVAR_GTFW_DEBUG_MESSAGES) {
			printl("give_tf_weapon ERROR: RegisterCustomWeapon failed. Returning null.")
		}
		return null
	}
	
	::TF_CUSTOM_WEAPONS_REGISTRY[item_name] <- TF_CUSTOM_WEPS(item_name, baseitem.className, baseitem.slot, baseitem.itemID, stats_function, custom_arms_model, baseitem.draw_seq, baseitem.ammo_type, baseitem.clip, baseitem.reserve, custom_weapon_model, custom_extra_wearable)
}

IncludeScript("give_tf_weapon_custom.nut")

/*//Dev Notes//
 - "m_bOnlyIterateItemViewAttributes" is used to mark if an item is custom. It's set to 1 via SetCustomWeapon.
 - "m_bValidatedAttachedEntity" is used for classes that have unintended weapons. It's set to 1 via AddWeapon.
DONE: Does not accout for weapons like mediguns, banners, shields, etc that add an extra wearable, so they aren't added to the player
NOTE: tf_weapon_builder lets you build on any class that has it
DONE: tf_weapon_builder can build sentries now
DONE: Consolidate think scripts from each weapon and put it on viewmodel (somehow)
DONE: Add think scripts for all that can take them.
DONE: Add more support for weapons with unique animations (like Kunai, Southern Hospitality, etc)
DONE: Remove "tf_weapon_" prefix from entities for easier computation (from TF_WEAPONS function at least)
DONE: Add class for all weapons containing various parameters (itemID, draw anim, primaryammo, secondaryammo, etc)
DONE: Add full names of all weapons (i.e. "Brass Beast")
DONE: Add feature for "GiveWeapon" to replace weapon if it fills a slot with another weapon in it. (It's now a cvar)
DONE: ReturnWeaponSlot
FIXME: Find a way to add/detect wearables like demoshield, booties, razorback, etc
DONE: Grab active weapon and switch it. If slot exists, just switch to the new weapon
FIXME: Fix crit boosts' colors
TODO: Giving class with passive items (Targe, Razorback, etc) need extra check when deleting secondaries because they're not in m_hMyWeapons
DONE: Update pistols and shotguns to give correct viewmodels (BUG: Scout can get Engineer's pistol)
DONE: Fix dropped weapons breaking the script
DONE: Add exceptions/checks for shotguns and pistols, katana, multiclass etc.
DONE: m_bValidatedAttachedEntity = weapon on unintended class
TODO: Keep weapons on resupply
DONE: Fix arms not updating when touching resupply if you have an unintended weapon
DONE: Thirdperson weapon model visibility bugs for custom weapons
DONE: Extra wearable on SetCustomWeapon
DONE: Search excemptions for custom weapons (so you can't delete the custom one if it uses the base of another)
DONE: ReactivateWeapon to fix disarmed weapons
DONE: Custom stats table auto-applied to custom weapons, based on a parameter set by SetCustomWeapon.
TODO: Add DeleteWeaponEx for searching all ents + ID value.
TODO: Add DisableWeaponEx, ReactivateWeaponEx
DONE: GiveWeapon not playing draw sequences for given weapon
TODO: m_bOnlyIterateItemViewAttributes sets to 1 on tf_dropped_weapon
WIP: Give weapon, set custom model, give another new weapon, first weapon resets arms
DONE?: Fix bug with GiveWeapon into SetCustomWeapon
DONE: Custom weapon database (basically just a table with stats)
DONE: RegisterCustomWeapon
DONE?: "misc" toolbox engineer fix
TODO: Example Hadouken fireball for Hot Hand

//::CVAR_GTFW_GIVEWEAPON_DROP_WEAPONS <- false		//Would drop your weapon in favor of the new one. Non-functional
//::CVAR_GTFW_USE_RESUPPLY_FIX	<- true		//Would fix resupply lockers destroying custom weapons. Non-functional

*/