//-----------------------------------------------------------------------------
// This file holds all custom weapon stuffs.
//-----------------------------------------------------------------------------
PrecacheModel("models/weapons/c_models/c_bigaxe/c_bigaxe.mdl")
PrecacheModel("models/weapons/c_models/c_big_mallet/c_big_mallet.mdl")
PrecacheModel("models/workshop/player/items/engineer/spr18_cold_case/spr18_cold_case.mdl")

enum GTFW_MODELS_CUSTOM_WEAPONS
{
	DEMO_BIGAXE		= "models/weapons/c_models/c_bigaxe/c_bigaxe.mdl",
	DEMO_BIGMALLET		= "models/weapons/c_models/c_big_mallet/c_big_mallet.mdl",
}

//-----------------------------------------------------------------------------
// Example weapon #1. This one is the Scout's Medieval Crossbow.
// It is based off the Crusader's Crossbow. Main stat thing is it slows the player while reloading.
//-----------------------------------------------------------------------------
function CW_Stats_Example_Scout_Medieval_Crossbow(weapon, player)
{
	weapon.AddAttribute("override projectile type", 8, -1)	//shoots Huntsman arrows (that headshot!)
	weapon.AddAttribute("damage penalty", 0.67, -1)		//reduces damage dealt
	weapon.AddAttribute("maxammo primary reduced", 0.45, -1)	//reduces primary ammo
	NetProps.SetPropIntArray(player, "localdata.m_iAmmo", 15, 1)	//sets reserve ammo of primary ammo type to 15
	weapon.AddAttribute("clip size upgrade atomic", 2, -1)	//increases max clipsize by +2
	weapon.SetClip1(3)	//sets clip to 3
	
//This is a think script. What it does is constantly runs in the background, checking for what ever we put in it.
	if( weapon.ValidateScriptScope() )
	{
		const THINK_SCOUTWEAPONRELOAD_DELAY = 0.25	//delay. Place at the end after `return` to change delay time. Default is 0.1.
		
		local entityscript = weapon.GetScriptScope()
		entityscript["think_SlowPlayerDuringReload"] <- function()	//Our script's name.
		{
			if ( NetProps.GetPropInt(weapon, "m_iReloadMode") == 2)	//if we are reloading...
			{
				player.AddCustomAttribute("move speed penalty", 0.67, -1)	//Add attribute that reduces move speed.
			}
			else if ( NetProps.GetPropInt(weapon, "m_iReloadMode") != 2)	//if -not- reloading...
			{
				player.RemoveCustomAttribute("move speed penalty")	//Remove move speed penalty.
			}
			return THINK_SCOUTWEAPONRELOAD_DELAY	//return to top after 0.25 ticks
		}
		AddThinkToEnt(weapon, "think_SlowPlayerDuringReload")	//Adds the think script to handle 'weapon' after it has been validated.
	}
}

//-----------------------------------------------------------------------------
//Registers this weapon as "Kritzader's Crossbow"...
// In this way, we can just use GiveWeapon() to add it to players, without all the hassle!
//-----------------------------------------------------------------------------
	RegisterCustomWeapon("Kritzader's Crossbow", "Crossbow", CW_Stats_Example_Scout_Medieval_Crossbow, null, GTFW_ARMS.MEDIC, "models/workshop/player/items/engineer/spr18_cold_case/spr18_cold_case.mdl")
	//First param: The weapon's name. Can be any string.
	//Second param: Weapon base. Accepts weapon string name (i.e. "Brass Beast"), item ID (i.e. 312), or handle.
	//Third param: Stats function. Executed when the weapon is given.
	//Fourth param: Custom model. Optional. Null in this case because it uses the original model.
	//Fifth param: Arms model. Optional. List in `give_tf_weapon.nut` under `enum GTFW_ARMS`.
	//Last param: Extra wearable. Optional. Adds another model onto the player, akin to Buff Banner, Vaccinator, etc.
	

//-----------------------------------------------------------------------------
// Example weapon #2. This one turns you into the Horseless Headless Horsemann.
// It is based off the Demo's Headtaker.
//-----------------------------------------------------------------------------
function CW_Stats_Example_Headless_Hatman(weapon, player)
{
	//player.SetCustomModelWithClassAnimations("models/bots/headless_hatman.mdl") //originally turned player into the actual Horsemann. Changed to use extra_wearable instead!
	NetProps.SetPropInt(player, "m_PlayerClass.m_iClass", 4)	//Forcefully sets tfclass to CIVILIAN
	
//Our stats
	weapon.ReapplyProvision()	//clears any stats cache on weapon
	weapon.AddAttribute("hidden maxhealth non buffed", (player.GetMaxHealth()*10) + 75, -1)	//using this attribute because it can't be overhealed
	player.SetHealth(player.GetMaxHealth())	//set health to max after given maxhealth bonus
	weapon.AddAttribute("move speed bonus", 1.25, -1)	//move speed up
	weapon.AddAttribute("damage bonus", 99, -1)	//instant death
	
	player.Weapon_Switch(weapon)	//vscript built-in function. Forces switch to handle 'weapon'.
	player.AddCond(41)	//locks player in melee
	
//Our effects
	player.SetForcedTauntCam(1)		//sets third person mode
	DoEntFire("!self", "SetModelScale", "1.4", 0, null, player)		//sets player scale
	Convars.SetValue("tf_item_based_forced_holiday", 2)		//sets holiday to use Halloween cosmetics
	player.AddCustomAttribute("SPELL: set Halloween footstep type", 2, -1)	//enables Horseless Horseshoes spell walk flames
	player.AddCustomAttribute("set item tint RGB", 9109759, -1)		//sets tint for Horseless Horseshoes
	
// Deletes all cosmetics from the player
	for (local i = 0; i < 42; i++)
	{
		local wearable = Entities.FindByClassname(null, "tf_*")

		if ( wearable != null && wearable.GetOwner() == this && wearable.GetClassname() != "tf_weapon_sword" && wearable.GetClassname() != "tf_viewmodel" && wearable.Name() != "tf_wearable_vscript" )
		{
			printl(wearable)
			wearable.Kill()
		}
		else
		{
			break
		}
	}
}

//-----------------------------------------------------------------------------
//Registers this weapon as "Horseless Headless Horsemann's Hell-Maker"...
//-----------------------------------------------------------------------------
	RegisterCustomWeapon("Horseless Headless Horsemann's Hell-Maker", "HHHH", CW_Stats_Example_Headless_Hatman, GTFW_MODELS_CUSTOM_WEAPONS.DEMO_BIGMALLET, null, "models/bots/headless_hatman.mdl")
	// Last param: Notice the extra_wearable is the Horseman's model.
